﻿using module ..\Include.psm1

param(
    [PSCustomObject]$Pools,
    [PSCustomObject]$Stats,
    [PSCustomObject]$Config,
    [PSCustomObject[]]$Devices
)

$Name = "$(Get-Item $MyInvocation.MyCommand.Path | Select-Object -ExpandProperty BaseName)"
$Path = ".\Bin\$($Name)\zjazz_amd.exe"
$HashSHA256 = "D35269E9EBF51DEA8AFFF2C3AFDC1633518D1A4689FA2CC6CA30425E1935E218"
$Uri = "https://github.com/zjazz/zjazz_amd_miner/releases/download/v0.92/zjazz_amd_win64_092.zip"
$ManualUri = "https://github.com/zjazz/zjazz_amd_miner"

$Miner_Config = Get-MinerConfig -Name $Name -Config $Config

$Commands = [PSCustomObject]@{ 
    "Phi2"     = " -a phi2"
    "Phi2-Lux" = " -a phi2"
}
#Commands from config file take precedence
if ($Miner_Config.Commands) { $Miner_Config.Commands | Get-Member -MemberType NoteProperty | Select-Object -ExpandProperty Name | ForEach-Object { $Commands | Add-Member $_ $($Miner_Config.Commands.$_) -Force } }

#CommonCommands from config file take precedence
if ($Miner_Config.CommonCommands) { $CommonCommands = $Miner_Config.CommonCommands = $Miner_Config.CommonCommands }
else { $CommonCommands = "" }

$Devices = @($Devices | Where-Object Type -EQ "GPU" | Where-Object Vendor -EQ "AMD" | Where-Object { $_.OpenCL.GlobalMemsize -ge 2GB } | Where-Object { $_.Model -match "^Baffin.*|^Ellesmere.*|^gfx900.*" })
$Devices | Select-Object Model -Unique | ForEach-Object { 
    $Miner_Device = @($Devices | Where-Object Model -EQ $_.Model)
    $Miner_Port = [UInt16]($Config.APIPort + ($Miner_Device | Select-Object -First 1 -ExpandProperty Id) + 1)

    $Commands | Get-Member -MemberType NoteProperty | Select-Object -ExpandProperty Name | ForEach-Object { $Algorithm_Norm = Get-Algorithm $_; $_ } | Where-Object { $Pools.$Algorithm_Norm.Protocol -eq "stratum+tcp" <#temp fix#> } | ForEach-Object { 
        $Miner_Name = (@($Name) + @($Miner_Device.Model | Sort-Object -unique | ForEach-Object { $Model = $_; "$(@($Miner_Device | Where-Object Model -eq $Model).Count)x$Model" }) | Select-Object) -join '-'

        #Get commands for active miner devices
        $Command = Get-CommandPerDevice -Command $Commands.$_ -DeviceIDs $Miner_Device.Type_Vendor_Index
        
        [PSCustomObject]@{ 
            Name                 = $Miner_Name
            DeviceName           = $Miner_Device.Name
            Path                 = $Path
            HashSHA256           = $HashSHA256
            Arguments            = ("$Command$CommonCommands -o $($Pools.PHI2.Protocol)://$($Pools.$Algorithm_Norm.Host):$($Pools.$Algorithm_Norm.Port) -u $($Pools.$Algorithm_Norm.User) -p $($Pools.$Algorithm_Norm.Pass) -d $(($Miner_Device | ForEach-Object { '{0:x}' -f $_.Type_Vendor_Index }) -join '')" -replace "\s+", " ").trim()
            HashRates            = [PSCustomObject]@{ $Algorithm_Norm = $Stats."$($Miner_Name)_$($Algorithm_Norm)_HashRate".Week }
            API                  = "Wrapper"
            Port                 = $Miner_Port
            URI                  = $Uri
            Fees                 = [PSCustomObject]@{ $Algorithm_Norm = 1 / 100 }
        }
    }
}
